/*

	These routines do the text-set-data I/O
	
	I'm implementing them as static function objects

*/


package tr4_textedit;

import java.awt.*;
import java.io.*;
import java.util.Vector;

import util.*;


final class TextSetIO {
	
	// Fixed in TR4
	static final byte XORByte = (byte)0xa5;

	// Read in and return whether it succeeded
	public static boolean Read(Frame Parent, InputStream IS, TextSet TS) {
		
		// Watch for I/O errors:
		try {
			// Read the overall numbers of stuff:
			byte[] InitBuffer = new byte[2*TextSet.NumTextBlocks];
			
			short[] NumStrings = new short[TextSet.NumTextBlocks];
			short TotalNumStrings = 0;
			short[] NumBytes = new short[TextSet.NumTextBlocks];
			short TotalNumBytes = 0;
			
			IS.read(InitBuffer);
			for (int i=0; i<TextSet.NumTextBlocks; i++)
				TotalNumStrings += (NumStrings[i] = Converter.GetLittleEndianShort(InitBuffer,2*i));
			
			IS.read(InitBuffer);
			for (int i=0; i<TextSet.NumTextBlocks; i++)
				TotalNumBytes += (NumBytes[i] = Converter.GetLittleEndianShort(InitBuffer,2*i));
			
			// Read in the offset tables as a single block:
			byte[] OffsetBuffer = new byte[2*TotalNumStrings];
			IS.read(OffsetBuffer);
			
			short[] OffsetTable = new short[TotalNumStrings+1];
			for (int i=0; i<TotalNumStrings; i++)
				OffsetTable[i] = Converter.GetLittleEndianShort(OffsetBuffer,2*i);
			OffsetTable[TotalNumStrings] = TotalNumBytes;
			
			// Read in the text data as a single block:
			byte[] TextBuffer = new byte[TotalNumBytes];
			IS.read(TextBuffer);
			
			short OffsetIndex = 0;
			for (int i=0; i<TextSet.NumTextBlocks; i++) {
				TextStringBlock TSB = TS.TextBlockSet[i];
				Vector Strings = (TSB.Strings = new Vector(NumStrings[i]));
				
				for (int j=0; j<NumStrings[i]; j++) {
					short Offset0 = OffsetTable[OffsetIndex];
					short Offset1 = OffsetTable[OffsetIndex+1];
					OffsetIndex++;
					
					for (int k=Offset0; k<Offset1-1; k++)
						TextBuffer[k] ^= XORByte;
					
					// Be sure to omit the C-string terminator byte
					String S = new String(TextBuffer, Offset0, Offset1-Offset0-1);
					Strings.addElement(S);
				}
			}
		} catch (Exception X) {
			InfoAlert A =
				new InfoAlert(Parent, "Read Error for " + TS.FileName, 480,72, X.toString());
			return false;
		}
		return true;
	}
	
	// Write out and return whether it succeeded
	public static boolean Write(Frame Parent, OutputStream OS, TextSet TS) {
		
		// Watch for I/O errors:
		try {
			// Write the overall numbers of stuff:
			byte[] InitBuffer = new byte[2*TextSet.NumTextBlocks];
			
			short[] NumStrings = new short[TextSet.NumTextBlocks];
			short TotalNumStrings = 0;
			short[] NumBytes = new short[TextSet.NumTextBlocks];
			short TotalNumBytes = 0;
			
			for (int i=0; i<TextSet.NumTextBlocks; i++) {
				Vector Strings = TS.TextBlockSet[i].Strings;
				TotalNumStrings += (NumStrings[i] = (short)Strings.size());
			}
			
			// While we are at it, set up the offset table here also
			short[] OffsetTable = new short[TotalNumStrings+1];
			short OffsetIndex = 0;
			short Offset = 0;
			for (int i=0; i<TextSet.NumTextBlocks; i++) {
				Vector Strings = TS.TextBlockSet[i].Strings;
				
				short NB = 0;
				for (int j=0; j<Strings.size(); j++) {
					String S = (String)Strings.elementAt(j);
				
					// Need to do it this way because a String character may be more than one byte
					// Also: need a C-string terminator byte at the end
					short NumStringBytes = (short)(S.getBytes().length + 1);
					
					OffsetTable[OffsetIndex++] = Offset;
					Offset += NumStringBytes;
					NB += NumStringBytes;
				}
				TotalNumBytes += (NumBytes[i] = NB);
			}
			OffsetTable[TotalNumStrings] = Offset;
			// Would-be assert
			if (Offset != TotalNumBytes)
				throw new Exception("Internal inconsistency: Offset != TotalNumBytes");
			
			for (int i=0; i<TextSet.NumTextBlocks; i++)
				Converter.PutLittleEndianShort(InitBuffer,2*i,NumStrings[i]);
			OS.write(InitBuffer);
			
			for (int i=0; i<TextSet.NumTextBlocks; i++)
				Converter.PutLittleEndianShort(InitBuffer,2*i,NumBytes[i]);
			OS.write(InitBuffer);
			
			// Write out the offset tables as a single block:
			byte[] OffsetBuffer = new byte[2*TotalNumStrings];
			for (int i=0; i<TotalNumStrings; i++)
				Converter.PutLittleEndianShort(OffsetBuffer,2*i,OffsetTable[i]);
			OS.write(OffsetBuffer);
			
			// Write out the text data as a single block:
			byte[] TextBuffer = new byte[TotalNumBytes];
			OffsetIndex = 0;
			for (int i=0; i<TextSet.NumTextBlocks; i++) {
				Vector Strings = TS.TextBlockSet[i].Strings;
				
				for (int j=0; j<Strings.size(); j++) {
					short Offset0 = OffsetTable[OffsetIndex];
					short Offset1 = OffsetTable[OffsetIndex+1];
					OffsetIndex++;
					
					String S = (String)Strings.elementAt(j);
					byte[] SBytes = S.getBytes();
					short NumSBytes = (short)SBytes.length;
					// Would-be assert
					if (NumSBytes != (Offset1-Offset0-1))
						throw new Exception("Internal inconsistency: NumSBytes != (from offsets)");
					
					for (int k=0; k<NumSBytes; k++)
						TextBuffer[Offset0+k] = (byte)(SBytes[k] ^ XORByte);
					
					// Need a C-string terminator byte at the end
					TextBuffer[Offset0+NumSBytes] = (byte)0;
				}
			}
			OS.write(TextBuffer);
			
		} catch (Exception X) {
			InfoAlert A =
				new InfoAlert(Parent, "Write Error for " + TS.FileName, 480,72, X.toString());
			return false;
		}
		return true;
	}
}
