// This is a dialog box for editing level scripts
// Set up not to dispose of itself when done, to avoid crashes in MW Java / MRJ
// This dialog box works in cooked mode, as opposed to raw mode;
// the difference is like the difference between assembly language (symbolic)
// and machine language (pure numeric).
// It assumes that a parsed level script passed to it
// is correctly parsed.

package tr_script;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import util.*;

final class CookedLevelScriptEditor extends Dialog {
	
	// Stuff that has to be local
	private Frame Parent;
	private Vector ParsedScriptVec;
	
	private List DispList;
	
	// Version that is edited
	private LevelScriptOp EditValue;
	
	// Stuff for editing the opcodes:
	// a popup menu, an opcode->menu-position table, and a menu-position->opcode table.
	private Choice OpcodeMenu;
	private short[] OpcodeToMenuLoc, MenuLocToOpcode;
	
	// Stuff for editing the operands:
	private TextField OperandField;
	
	// Must be called whenever the opcode has changed;
	// the operand's significance depends on the opcode.
	private void UpdateOperand() {
		OperandField.setText(Short.toString(EditValue.Operand));
		LevelScriptParseInfo ParseInfo = LevelScriptParseInfo.Do(EditValue.Opcode);
		OperandField.setEnabled(ParseInfo.NumOperands >= 1);
	}
	
	// Create a string in a suitable format for the display list:
	private String MakeDispString(int ic, LevelScriptOp Value) {
		String OutStr = Integer.toString(ic);
		LevelScriptParseInfo ParseInfo = LevelScriptParseInfo.Do(Value.Opcode);
		OutStr = OutStr + ": " + ParseInfo.Description;
		if (ParseInfo.NumOperands >= 1)
			OutStr = OutStr + ": " + Value.Operand;
		return OutStr;
	}
	
	// Rebuild the display list
	private void MakeDispList() {
		DispList.removeAll();
		for (int ic=0; ic<ParsedScriptVec.size(); ic++) {
			LevelScriptOp Value = (LevelScriptOp)ParsedScriptVec.elementAt(ic);
			DispList.add(MakeDispString(ic,Value));
		}
	}
	
	CookedLevelScriptEditor(Frame _Parent, Vector _ParsedScriptVec, String Name) {
		super(_Parent,Name,true);
		setSize(640,384);
		setResizable(false);
		
		// Necessary for some of the inner classes
		Parent = _Parent;
		ParsedScriptVec = _ParsedScriptVec;
		
		// Starting object to be edited
		EditValue = new LevelScriptOp();
		EditValue.Opcode = 9;
		EditValue.Operand = 0;
		
		// The layout
		GridBagLayout L = new GridBagLayout();
		setLayout(L);
		GridBagConstraints C = new GridBagConstraints();
				
		// Padding
		C.insets = new Insets(2,2,2,2);
		
		// The display list itself
		DispList = new List(16);
		MakeDispList();
		
		// Action is to get a string from the list
		DispList.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				EditValue = new LevelScriptOp((LevelScriptOp)ParsedScriptVec.elementAt(ic));
				OpcodeMenu.select(OpcodeToMenuLoc[EditValue.Opcode]);
				UpdateOperand();
			}}
		);
		C.gridx = 0;
		C.gridheight = 12;
		C.anchor = GridBagConstraints.NORTHWEST;
		add(DispList,C);
		C.gridheight = 1;
		
		int Row = 0;
		C.gridx = 1;
		C.gridy = (Row++);
		C.anchor = GridBagConstraints.WEST;
		add(new Label("Opcode:"),C);
		
		// Popup menu for the opcodes
		OpcodeMenu = new Choice();
		OpcodeToMenuLoc = new short[LevelScriptParseInfo.MaxOpcode() + 1];
		short Indx = 0;
		for (short k=0; k<OpcodeToMenuLoc.length; k++) {
			LevelScriptParseInfo ParseInfo = LevelScriptParseInfo.Do(k);
			if (ParseInfo != null) {
				OpcodeToMenuLoc[k] = Indx++;
				OpcodeMenu.addItem(ParseInfo.Description);
			} else {
				OpcodeToMenuLoc[k] = -1;
				continue;
			}
		}
		MenuLocToOpcode = new short[Indx];
		for (short k=0; k<OpcodeToMenuLoc.length; k++) {
			short Loc = OpcodeToMenuLoc[k];
			if (Loc < 0) continue;
			MenuLocToOpcode[Loc] = k;
		}
		OpcodeMenu.addItemListener(new ItemListener()
			{public void itemStateChanged(ItemEvent E) {
				int ic = OpcodeMenu.getSelectedIndex();
				if (ic < 0) return;
				EditValue.Opcode = MenuLocToOpcode[ic];
				UpdateOperand();
			}}
		);
		OpcodeMenu.select(OpcodeToMenuLoc[EditValue.Opcode]);
		
		C.gridy = (Row++);
		add(OpcodeMenu,C);
		
		C.gridy = (Row++);
		add(new Label("Operand:"),C);
		
		// The operand field
		OperandField = new TextField("",24);
		OperandField.addTextListener(new TextListener()
			{public void textValueChanged(TextEvent E) {
				try {
					EditValue.Operand = Short.parseShort(OperandField.getText());
				} catch(Exception X) {
					OperandField.setText(Short.toString(EditValue.Operand));
				}
			}}
		);
		UpdateOperand();
		
		C.gridy = (Row++);
		add(OperandField,C);
		
		// The buttons:
		C.fill = GridBagConstraints.HORIZONTAL;
		
		// Get the currently-selected value
		Button GetButton = new Button("Get from List");
		GetButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				EditValue = new LevelScriptOp((LevelScriptOp)ParsedScriptVec.elementAt(ic));
				OpcodeMenu.select(OpcodeToMenuLoc[EditValue.Opcode]);
				UpdateOperand();
			}}
		);
		C.gridy = (Row++);
		add(GetButton,C);
		
		// Set the currently-selected value to the text-field contents
		Button ChangeButton = new Button("Change in List");
		ChangeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				ParsedScriptVec.setElementAt(new LevelScriptOp(EditValue),ic);
				DispList.replaceItem(MakeDispString(ic,EditValue),ic);
					
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(ChangeButton,C);
		
		// Insert a value before the currently-selected value location
		Button InsertBeforeButton = new Button("Insert Before");
		InsertBeforeButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (ParsedScriptVec.size() <= 0) {
					ParsedScriptVec.addElement(new LevelScriptOp(EditValue));
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					ParsedScriptVec.insertElementAt(new LevelScriptOp(EditValue),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertBeforeButton,C);
		
		// Insert a value after the currently-selected value location
		Button InsertAfterButton = new Button("Insert After");
		InsertAfterButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic;
				if (ParsedScriptVec.size() <= 0) {
					ParsedScriptVec.addElement(new LevelScriptOp(EditValue));
					ic = 0;
				} else {
					ic = DispList.getSelectedIndex();
					if (ic < 0) return;
					ic++;
					ParsedScriptVec.insertElementAt(new LevelScriptOp(EditValue),ic);
				}
				
				MakeDispList();
				DispList.select(ic);
				
				// Resize if necessary
				validate();
				setSize(getPreferredSize());
				validate();
			}}
		);
		C.gridy = (Row++);
		add(InsertAfterButton,C);

		// Delete the currently-selected string
		Button DeleteButton = new Button("Delete");
		DeleteButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E) {
				int ic = DispList.getSelectedIndex();
				if (ic < 0) return;
				
				String WhichOne = MakeDispString(ic,(LevelScriptOp)ParsedScriptVec.elementAt(ic));
				WarningAlert A = new WarningAlert(Parent,"Warning!",384,72,
					"Really want to delete \"" + WhichOne + "\"?");
				if (A.If_OK) {
					ParsedScriptVec.removeElementAt(ic);
					MakeDispList();
					DispList.select(ic);
				
					// Resize if necessary
					validate();
					setSize(getPreferredSize());
					validate();
				}
			}}
		);
		C.gridy = (Row++);
		add(DeleteButton,C);
		
		Button CloseButton = new Button("Close");
		CloseButton.addActionListener(new ActionListener()
			{public void actionPerformed(ActionEvent E)
				{/*dispose();*/setVisible(false);
			}}
		);
		C.gridy = (Row++);
		add(CloseButton,C);
								
		validate();
		setVisible(true);
		
		setSize(getPreferredSize());
		validate();
	}
}
