/*
	This class contains methods for getting and putting various kinds of objects:
	big- and little-endian integers with varying lengths, and string
*/


package util;

public final class Converter {
	
	/*
		Read and write with specified endianness;
		I have no routines dealing with longs (64-bit integers),
		because these values are not often stored
	*/
	
	public static final short ShortByteMask = 0x00ff;
	public static final int IntByteMask = 0x000000ff;

	public static short GetBigEndianShort(byte []BArray, int Offset) {
		short val = 0;
		for (int q=0; q<2; q++) {
			short b = BArray[Offset+q];
			val |= (b & ShortByteMask) << ((1-q) << 3);
		}
		return val;
	}

	public static short GetLittleEndianShort(byte []BArray, int Offset) {
		short val = 0;
		for (int q=0; q<2; q++) {
			short b = BArray[Offset+q];
			val |= (b & ShortByteMask) << (q << 3);
		}
		return val;
	}

	public static int GetBigEndianInt(byte []BArray, int Offset) {
		int val = 0;
		for (int q=0; q<4; q++) {
			int b = BArray[Offset+q];
			val |= (b & IntByteMask) << ((3-q) << 3);
		}
		return val;
	}

	public static int GetLittleEndianInt(byte []BArray, int Offset) {
		int val = 0;
		for (int q=0; q<4; q++) {
			int b = BArray[Offset+q];
			val |= (b & IntByteMask) << (q << 3);
		}
		return val;
	}

	public static void PutBigEndianShort(byte []BArray, int Offset, short val) {
		for (int q=0; q<2; q++) {
			BArray[Offset+q] = (byte)(val >> ((1-q) << 3));
		}
	}

	public static void PutLittleEndianShort(byte []BArray, int Offset, short val) {
		for (int q=0; q<2; q++) {
			BArray[Offset+q] = (byte)(val >> (q << 3));
		}
	}

	public static void PutBigEndianInt(byte []BArray, int Offset, int val) {
		for (int q=0; q<4; q++) {
			BArray[Offset+q] = (byte)(val >> ((1-q) << 3));
		}
	}

	public static void PutLittleEndianInt(byte []BArray, int Offset, int val) {
		for (int q=0; q<4; q++) {
			BArray[Offset+q] = (byte)(val >> (q << 3));
		}
	}
	
	/*
		Convert byte strings to Java character strings and vice versa
	*/
	
	// This class is designed for reading delimited strings such as C strings
	public static String GetCharsDelimited(byte []BArray, int Offset, byte Delimiter) {
		int Length = 0;
		for (int q=Offset; q<BArray.length; q++) {
			if (BArray[q] == Delimiter) {
				Length = q - Offset;
			}
		}
		
		return new String(BArray,Offset,Length);
	}
	
	// This class is designed to put a string into a fixed-size array
	public static void PutCharsFixed(byte []BArray, int Offset, int TotalLength, String OutStr) {
		byte[] OutArray = OutStr.getBytes();
		
		int NBytes;
		if (OutStr.length() < TotalLength) {
			NBytes = OutStr.length();
			// Pad with zeros
			for (int q=OutStr.length(); q<TotalLength; q++) {
				BArray[Offset+q] = 0;
			}
		} else {
			NBytes = TotalLength;
		}
		for (int q=0; q<NBytes; q++)
			BArray[Offset+q] = OutArray[q];
	}
	
	/*
		Extract and place byte strings:
	*/
	
	public static byte[] GetBytes(byte []BArray, int Offset, int Length) {
		byte[] OutBytes = new byte[Length];
		for (int q=0; q<Length; q++)
			OutBytes[q] = BArray[Offset+q];
		
		return OutBytes;
	}
	
	public static void PutBytes(byte []BArray, int Offset, byte[] OutBytes) {
		for (int q=0; q<OutBytes.length; q++)
			BArray[Offset+q] = OutBytes[q];
	}
}